@@ -4,6 +4,10 @@ Text Markup
 
 .. versionadded:: 1.1.0
 
+.. versionchanged:: 1.10.1
+
+    Added `font_context`, `font_features` and `text_language` (Pango only)
+
 We provide a simple text-markup for inline text styling. The syntax look the
 same as the `BBCode <http://en.wikipedia.org/wiki/BBCode>`_.
 
@@ -23,7 +27,17 @@ The following tags are available:
 ``[s][/s]``
     Strikethrough text
 ``[font=<str>][/font]``
-    Change the font
+    Change the font (note: this refers to a TTF file or registered alias)
+``[font_context=<str>][/font_context]``
+    Change context for the font, use string value "none" for isolated context.
+``[font_family=<str>][/font_family]``
+    Font family to request for drawing. This is only valid when using a
+    font context, and takes precedence over `[font]`. See
+    :class:`kivy.uix.label.Label` for details.
+``[font_features=<str>][/font_features]``
+    OpenType font features, in CSS format, this is passed straight
+    through to Pango. The effects of requesting a feature depends on loaded
+    fonts, library versions, etc. Pango only, requires v1.38 or later.
 ``[size=<size>][/size]``
     Change the font size. <size> should be an integer, optionally with a
     unit (i.e. ``16sp``)
@@ -39,6 +53,12 @@ The following tags are available:
     Display the text at a subscript position relative to the text before it.
 ``[sup][/sup]``
     Display the text at a superscript position relative to the text before it.
+``[text_language=<str>][/text_language]``
+    Language of the text, this is an RFC-3066 format language tag (as string),
+    for example "en_US", "zh_CN", "fr" or "ja". This can impact font selection,
+    metrics and rendering. For example, the same bytes of text can look
+    different for `ur` and `ar` languages, though both use Arabic script.
+    Use the string `'none'` to revert to locale detection. Pango only.
 
 If you need to escape the markup from the current text, use
 :func:`kivy.utils.escape_markup`.
@@ -64,7 +84,7 @@ if Label is None:
 class MarkupLabel(MarkupLabelBase):
     '''Markup text label.
 
-    See module documentation for more informations.
+    See module documentation for more information.
     '''
 
     def __init__(self, *largs, **kwargs):
@@ -93,13 +113,13 @@ class MarkupLabel(MarkupLabelBase):
 
     @property
     def markup(self):
-        '''Return the text with all the markup splitted::
+        '''Return the text with all the markup split::
 
             >>> MarkupLabel('[b]Hello world[/b]').markup
             >>> ('[b]', 'Hello world', '[/b]')
 
         '''
-        s = re.split('(\[.*?\])', self.label)
+        s = re.split(r'(\[.*?\])', self.label)
         s = [x for x in s if x != '']
         return s
 
@@ -136,7 +156,7 @@ class MarkupLabel(MarkupLabelBase):
         uw, uh = self.text_size
         spush = self._push_style
         spop = self._pop_style
-        opts = options = self.options
+        options = self.options
         options['_ref'] = None
         options['_anchor'] = None
         options['script'] = 'normal'
@@ -145,10 +165,13 @@ class MarkupLabel(MarkupLabelBase):
         # flattened later when shortening and broken up lines if broken
         # mid-word will have space mid-word when lines are joined
         uw_temp = None if shorten else uw
-        xpad = options['padding_x']
+        xpad = options['padding'][0] + options['padding'][2]
         uhh = (None if uh is not None and options['valign'] != 'top' or
                options['shorten'] else uh)
         options['strip'] = options['strip'] or options['halign'] == 'justify'
+        find_base_dir = Label.find_base_direction
+        base_dir = options['base_direction']
+        self._resolved_base_dir = None
         for item in self.markup:
             if item == '[b]':
                 spush('bold')
@@ -206,6 +229,32 @@ class MarkupLabel(MarkupLabelBase):
             elif item == '[/font]':
                 spop('font_name')
                 self.resolve_font_name()
+            elif item[:13] == '[font_family=':
+                spush('font_family')
+                options['font_family'] = item[13:-1]
+            elif item == '[/font_family]':
+                spop('font_family')
+            elif item[:14] == '[font_context=':
+                fctx = item[14:-1]
+                if not fctx or fctx.lower() == 'none':
+                    fctx = None
+                spush('font_context')
+                options['font_context'] = fctx
+            elif item == '[/font_context]':
+                spop('font_context')
+            elif item[:15] == '[font_features=':
+                spush('font_features')
+                options['font_features'] = item[15:-1]
+            elif item == '[/font_features]':
+                spop('font_features')
+            elif item[:15] == '[text_language=':
+                lang = item[15:-1]
+                if not lang or lang.lower() == 'none':
+                    lang = None
+                spush('text_language')
+                options['text_language'] = lang
+            elif item == '[/text_language]':
+                spop('text_language')
             elif item[:5] == '[sub]':
                 spush('font_size')
                 spush('script')
@@ -233,6 +282,8 @@ class MarkupLabel(MarkupLabelBase):
             elif not clipped:
                 item = item.replace('&bl;', '[').replace(
                     '&br;', ']').replace('&amp;', '&')
+                if not base_dir:
+                    base_dir = self._resolved_base_dir = find_base_dir(item)
                 opts = copy(options)
                 extents = self.get_cached_extents()
                 opts['space_width'] = extents(' ')[0]
@@ -287,7 +338,7 @@ class MarkupLabel(MarkupLabelBase):
             # XXX: update refs to justified pos
             # when justify, each line should've been stripped already
             split = partial(re.split, re.compile('( +)'))
-            uww = uw - 2 * xpad
+            uww = uw - xpad
             chr = type(self.text)
             space = chr(' ')
             empty = chr('')
@@ -402,19 +453,28 @@ class MarkupLabel(MarkupLabelBase):
         return int(w), int(h)
 
     def render_lines(self, lines, options, render_text, y, size):
-        xpad = options['padding_x']
+        padding_left = options['padding'][0]
+        padding_right = options['padding'][2]
         w = size[0]
         halign = options['halign']
         refs = self._refs
         anchors = self._anchors
+        base_dir = options['base_direction'] or self._resolved_base_dir
+        auto_halign_r = halign == 'auto' and base_dir and 'rtl' in base_dir
 
         for layout_line in lines:  # for plain label each line has only one str
             lw, lh = layout_line.w, layout_line.h
-            x = xpad
+            x = padding_left
             if halign == 'center':
-                x = int((w - lw) / 2.)
-            elif halign == 'right':
-                x = max(0, int(w - lw - xpad))
+                x = min(
+                    int(w - lw),
+                    max(
+                        int(padding_left),
+                        int((w - lw + padding_left - padding_right) / 2.0)
+                    )
+                )
+            elif halign == 'right' or auto_halign_r:
+                x = max(0, int(w - lw - padding_right))
             layout_line.x = x
             layout_line.y = y
             psp = pph = 0
@@ -616,12 +676,18 @@ class MarkupLabel(MarkupLabelBase):
         if uw is None:
             return w, h, lines
         old_opts = copy(self.options)
-        uw = max(0, int(uw - old_opts['padding_x'] * 2 - margin))
+        uw = max(
+            0,
+            int(uw - old_opts["padding"][0] - old_opts["padding"][2] - margin),
+        )
         chr = type(self.text)
         ssize = textwidth(' ')
         c = old_opts['split_str']
         line_height = old_opts['line_height']
-        xpad, ypad = old_opts['padding_x'], old_opts['padding_y']
+        xpad, ypad = (
+            old_opts["padding"][0] + old_opts["padding"][2],
+            old_opts["padding"][1] + old_opts["padding"][3],
+        )
         dir = old_opts['shorten_from'][0]
 
         # flatten lines into single line
@@ -643,8 +709,8 @@ class MarkupLabel(MarkupLabelBase):
             lh = max([word.lh for word in line] + [0]) * line_height
             self.is_shortened = False
             return (
-                lw + 2 * xpad,
-                lh + 2 * ypad,
+                lw + xpad,
+                lh + ypad,
                 [LayoutLine(0, 0, lw, lh, 1, 0, line)]
             )
 
@@ -661,8 +727,8 @@ class MarkupLabel(MarkupLabelBase):
             s = textwidth('..')
             if s[0] <= uw:
                 return (
-                    s[0] + 2 * xpad,
-                    s[1] * line_height + 2 * ypad,
+                    s[0] + xpad,
+                    s[1] * line_height + ypad,
                     [LayoutLine(
                         0, 0, s[0], s[1], 1, 0,
                         [LayoutWord(old_opts, s[0], s[1], '..')])]
@@ -671,8 +737,8 @@ class MarkupLabel(MarkupLabelBase):
             else:
                 s = textwidth('.')
                 return (
-                    s[0] + 2 * xpad,
-                    s[1] * line_height + 2 * ypad,
+                    s[0] + xpad,
+                    s[1] * line_height + ypad,
                     [LayoutLine(
                         0, 0, s[0], s[1], 1, 0,
                         [LayoutWord(old_opts, s[0], s[1], '.')])]
@@ -714,8 +780,8 @@ class MarkupLabel(MarkupLabelBase):
                 self.options = old_opts
                 self.is_shortened = True
                 return (
-                    lw + 2 * xpad,
-                    lh + 2 * ypad,
+                    lw + xpad,
+                    lh + ypad,
                     [LayoutLine(0, 0, lw, lh, 1, 0, line1)]
                 )
 
@@ -777,8 +843,8 @@ class MarkupLabel(MarkupLabelBase):
                 self.options = old_opts
                 self.is_shortened = True
                 return (
-                    lw + 2 * xpad,
-                    lh + 2 * ypad,
+                    lw + xpad,
+                    lh + ypad,
                     [LayoutLine(0, 0, lw, lh, 1, 0, line1)]
                 )
 
@@ -820,7 +886,7 @@ class MarkupLabel(MarkupLabelBase):
         if uw < lw:
             self.is_shortened = True
         return (
-            lw + 2 * xpad,
-            lh + 2 * ypad,
+            lw + xpad,
+            lh + ypad,
             [LayoutLine(0, 0, lw, lh, 1, 0, line1)]
         )
