@@ -22,13 +22,6 @@ except ImportError:
 class LabelSDL2(LabelBase):
 
     def _get_font_id(self):
-        if PY2:
-            try:
-                return '|'.join([unicode(self.options[x]) for x
-                    in ('font_size', 'font_name_r', 'bold',
-                        'italic', 'underline', 'strikethrough')])
-            except UnicodeDecodeError:
-                pass
         return '|'.join([str(self.options[x]) for x
             in ('font_size', 'font_name_r', 'bold',
                 'italic', 'underline', 'strikethrough')])
