@@ -1,4 +1,81 @@
-'''See :class:`FocusBehavior` for more details.
+'''
+Focus Behavior
+==============
+
+The :class:`~kivy.uix.behaviors.FocusBehavior`
+`mixin <https://en.wikipedia.org/wiki/Mixin>`_ class provides
+keyboard focus behavior. When combined with other
+FocusBehavior widgets it allows one to cycle focus among them by pressing
+tab. In addition, upon gaining focus, the instance will automatically
+receive keyboard input.
+
+Focus, very different from selection, is intimately tied with the keyboard;
+each keyboard can focus on zero or one widgets, and each widget can only
+have the focus of one keyboard. However, multiple keyboards can focus
+simultaneously on different widgets. When escape is hit, the widget having
+the focus of that keyboard will de-focus.
+
+Managing focus
+--------------
+
+In essence, focus is implemented as a doubly linked list, where each
+node holds a (weak) reference to the instance before it and after it,
+as visualized when cycling through the nodes using tab (forward) or
+shift+tab (backward). If a previous or next widget is not specified,
+:attr:`focus_next` and :attr:`focus_previous` defaults to `None`. This
+means that the :attr:`~kivy.uix.widget.Widget.children` list and
+:attr:`parents <kivy.uix.widget.Widget.parent>` are
+walked to find the next focusable widget, unless :attr:`focus_next` or
+:attr:`focus_previous` is set to the `StopIteration` class, in which case
+focus stops there.
+
+For example, to cycle focus between :class:`~kivy.uix.button.Button`
+elements of a :class:`~kivy.uix.gridlayout.GridLayout`::
+
+    class FocusButton(FocusBehavior, Button):
+      pass
+
+    grid = GridLayout(cols=4)
+    for i in range(40):
+        grid.add_widget(FocusButton(text=str(i)))
+    # clicking on a widget will activate focus, and tab can now be used
+    # to cycle through
+
+When using a software keyboard, typical on mobile and touch devices, the
+keyboard display behavior is determined by the
+:attr:`~kivy.core.window.WindowBase.softinput_mode` property. You can use
+this property to ensure the focused widget is not covered or obscured by the
+keyboard.
+
+Initializing focus
+------------------
+
+Widgets needs to be visible before they can receive the focus. This means that
+setting their *focus* property to True before they are visible will have no
+effect. To initialize focus, you can use the 'on_parent' event::
+
+    from kivy.app import App
+    from kivy.uix.textinput import TextInput
+
+    class MyTextInput(TextInput):
+        def on_parent(self, widget, parent):
+            self.focus = True
+
+    class SampleApp(App):
+        def build(self):
+            return MyTextInput()
+
+    SampleApp().run()
+
+If you are using a :class:`~kivy.uix.popup`, you can use the 'on_open' event.
+
+For an overview of behaviors, please refer to the :mod:`~kivy.uix.behaviors`
+documentation.
+
+.. warning::
+
+    This code is still experimental, and its API is subject to change in a
+    future version.
 '''
 
 __all__ = ('FocusBehavior', )
@@ -17,53 +94,14 @@ if Config:
 
 
 class FocusBehavior(object):
-    '''This `mixin <https://en.wikipedia.org/wiki/Mixin>`_ class provides
-    keyboard focus behavior. When combined with other
+    '''Provides keyboard focus behavior. When combined with other
     FocusBehavior widgets it allows one to cycle focus among them by pressing
-    tab. In addition, upon gaining focus, the instance will automatically
-    receive keyboard input.
-
-    Focus, very different from selection, is intimately tied with the keyboard;
-    each keyboard can focus on zero or one widgets, and each widget can only
-    have the focus of one keyboard. However, multiple keyboards can focus
-    simultaneously on different widgets. When escape is hit, the widget having
-    the focus of that keyboard will de-focus.
-
-    In essence, focus is implemented as a doubly linked list, where each
-    node holds a (weak) reference to the instance before it and after it,
-    as visualized when cycling through the nodes using tab (forward) or
-    shift+tab (backward). If a previous or next widget is not specified,
-    :attr:`focus_next` and :attr:`focus_previous` defaults to `None`. This
-    means that the :attr:`~kivy.uix.widget.Widget.children` list and
-    :attr:`parents <kivy.uix.widget.Widget.parent>` are
-    walked to find the next focusable widget, unless :attr:`focus_next` or
-    :attr:`focus_previous` is set to the `StopIteration` class, in which case
-    focus stops there.
-
-    For example, to cycle focus between :class:`~kivy.uix.button.Button`
-    elements of a :class:`~kivy.uix.gridlayout.GridLayout`::
-
-        class FocusButton(FocusBehavior, Button):
-            pass
-
-        grid = GridLayout(cols=4)
-        for i in range(40):
-            grid.add_widget(FocusButton(text=str(i)))
-        # clicking on a widget will activate focus, and tab can now be used
-        # to cycle through
-
-    When using a software keyboard, typical on mobile and touch devices, the
-    keyboard display behavior is determined by the
-    :attr:`~kivy.core.window.WindowBase.softinput_mode` property. You can use
-    this property to ensure the focused widget is not covered or obscured by the
-    keyboard.
+    tab. Please see the
+    :mod:`focus behavior module documentation <kivy.uix.behaviors.focus>`
+    for more information.
 
     .. versionadded:: 1.9.0
 
-    .. warning::
-
-        This code is still experimental, and its API is subject to change in a
-        future version.
     '''
 
     _requested_keyboard = False
@@ -95,7 +133,7 @@ class FocusBehavior(object):
             self.focus = False    # this'll unbind
             if self._keyboard:  # remove assigned keyboard from dict
                 del keyboards[keyboard]
-        if value and not value in keyboards:
+        if value and value not in keyboards:
             keyboards[value] = None
         self._keyboard = value
         self.focus = focus
@@ -184,8 +222,8 @@ class FocusBehavior(object):
     :attr:`~kivy.core.window.WindowBase.softinput_mode` property to determine
     how the keyboard display is handled.
 
-    :attr:`focus` is a :class:`~kivy.properties.BooleanProperty` and defaults to
-    False.
+    :attr:`focus` is a :class:`~kivy.properties.BooleanProperty` and defaults
+    to False.
     '''
 
     focused = focus
@@ -199,24 +237,44 @@ class FocusBehavior(object):
         2.0.0.
     '''
 
+    keyboard_suggestions = BooleanProperty(True)
+    '''If True provides auto suggestions on top of keyboard.
+    This will only work if :attr:`input_type` is set to `text`, `url`, `mail` or
+    `address`.
+
+    .. warning::
+        On Android, `keyboard_suggestions` relies on
+        `InputType.TYPE_TEXT_FLAG_NO_SUGGESTIONS` to work, but some keyboards
+        just ignore this flag. If you want to disable suggestions at all on
+        Android, you can set `input_type` to `null`, which will request the
+        input method to run in a limited "generate key events" mode.
+
+    .. versionadded:: 2.1.0
+
+    :attr:`keyboard_suggestions` is a :class:`~kivy.properties.BooleanProperty`
+    and defaults to True
+    '''
+
     def _set_on_focus_next(self, instance, value):
-        ''' If changing code, ensure following code is not infinite loop:
+        '''If changing focus, ensure your code does not create an infinite loop.
+        eg:
+        ```python
         widget.focus_next = widget
         widget.focus_previous = widget
-        widget.focus_previous = widget2
+        ```
         '''
-        next = self._old_focus_next
-        if next is value:   # prevent infinite loop
+        next_ = self._old_focus_next
+        if next_ is value:   # prevent infinite loop
             return
 
-        if isinstance(next, FocusBehavior):
-            next.focus_previous = None
+        if isinstance(next_, FocusBehavior):
+            next_.focus_previous = None
         self._old_focus_next = value
         if value is None or value is StopIteration:
             return
         if not isinstance(value, FocusBehavior):
-            raise ValueError('focus_next accepts only objects based'
-                             ' on FocusBehavior, or the `StopIteration` class.')
+            raise ValueError('focus_next accepts only objects based on'
+                             ' FocusBehavior, or the `StopIteration` class.')
         value.focus_previous = self
 
     focus_next = ObjectProperty(None, allownone=True)
@@ -257,7 +315,7 @@ class FocusBehavior(object):
             return
         if not isinstance(value, FocusBehavior):
             raise ValueError('focus_previous accepts only objects based'
-                             ' on FocusBehavior, or the `StopIteration` class.')
+                             'on FocusBehavior, or the `StopIteration` class.')
         value.focus_next = self
 
     focus_previous = ObjectProperty(None, allownone=True)
@@ -295,15 +353,22 @@ class FocusBehavior(object):
     defaults to 'auto'. Can be one of 'auto' or 'managed'.
     '''
 
-    input_type = OptionProperty('text', options=('text', 'number', 'url',
-                                                 'mail', 'datetime', 'tel',
-                                                 'address'))
+    input_type = OptionProperty('null', options=('null', 'text', 'number',
+                                                 'url', 'mail', 'datetime',
+                                                 'tel', 'address'))
     '''The kind of input keyboard to request.
 
     .. versionadded:: 1.8.0
 
+    .. versionchanged:: 2.1.0
+        Changed default value from `text` to `null`. Added `null` to options.
+
+        .. warning::
+            As the default value has been changed, you may need to adjust
+            `input_type` in your code.
+
     :attr:`input_type` is an :class:`~kivy.properties.OptionsProperty` and
-    defaults to 'text'. Can be one of 'text', 'number', 'url', 'mail',
+    defaults to 'null'. Can be one of 'null', 'text', 'number', 'url', 'mail',
     'datetime', 'tel' or 'address'.
     '''
 
@@ -312,11 +377,10 @@ class FocusBehavior(object):
     '''Whether a instance should lose focus when clicked outside the instance.
 
     When a user clicks on a widget that is focus aware and shares the same
-    keyboard as this widget (which in the case of only one keyboard, are
-    all focus aware widgets), then as the other widgets gains focus, this
-    widget loses focus. In addition to that, if this property is `True`,
-    clicking on any widget other than this widget, will remove focus form this
-    widget.
+    keyboard as this widget (which in the case with only one keyboard),
+    then as the other widgets gain focus, this widget loses focus. In addition
+    to that, if this property is `True`, clicking on any widget other than this
+    widget, will remove focus from this widget.
 
     :attr:`unfocus_on_touch` is a :class:`~kivy.properties.BooleanProperty` and
     defaults to `False` if the `keyboard_mode` in :attr:`~kivy.config.Config`
@@ -341,7 +405,7 @@ class FocusBehavior(object):
             self.focus = False
 
     def _on_focus(self, instance, value, *largs):
-        if self.is_focusable and self.keyboard_mode == 'auto':
+        if self.keyboard_mode == 'auto':
             if value:
                 self._bind_keyboard()
             else:
@@ -350,9 +414,12 @@ class FocusBehavior(object):
     def _ensure_keyboard(self):
         if self._keyboard is None:
             self._requested_keyboard = True
-            keyboard = self._keyboard =\
-                EventLoop.window.request_keyboard(
-                    self._keyboard_released, self, input_type=self.input_type)
+            keyboard = self._keyboard = EventLoop.window.request_keyboard(
+                self._keyboard_released,
+                self,
+                input_type=self.input_type,
+                keyboard_suggestions=self.keyboard_suggestions,
+            )
             keyboards = FocusBehavior._keyboards
             if keyboard not in keyboards:
                 keyboards[keyboard] = None
@@ -422,7 +489,7 @@ class FocusBehavior(object):
 
     def _get_focus_next(self, focus_dir):
         current = self
-        walk_tree = 'walk' if focus_dir is 'focus_next' else 'walk_reverse'
+        walk_tree = 'walk' if focus_dir == 'focus_next' else 'walk_reverse'
 
         while 1:
             # if we hit a focusable, walk through focus_xxx
@@ -435,7 +502,7 @@ class FocusBehavior(object):
 
             # hit unfocusable, walk widget tree
             itr = getattr(current, walk_tree)(loopback=True)
-            if focus_dir is 'focus_next':
+            if focus_dir == 'focus_next':
                 next(itr)  # current is returned first  when walking forward
             for current in itr:
                 if isinstance(current, FocusBehavior):
@@ -449,6 +516,20 @@ class FocusBehavior(object):
             else:
                 return None
 
+    def get_focus_next(self):
+        '''Returns the next focusable widget using either :attr:`focus_next`
+           or the :attr:`children` similar to the order when tabbing forwards
+           with the ``tab`` key.
+        '''
+        return self._get_focus_next('focus_next')
+
+    def get_focus_previous(self):
+        '''Returns the previous focusable widget using either
+           :attr:`focus_previous` or the :attr:`children` similar to the
+           order when the ``tab`` + ``shift`` keys are triggered together.
+        '''
+        return self._get_focus_next('focus_previous')
+
     def keyboard_on_key_down(self, window, keycode, text, modifiers):
         '''The method bound to the keyboard when the instance has focus.
 
@@ -465,10 +546,13 @@ class FocusBehavior(object):
         key was consumed.
         '''
         if keycode[1] == 'tab':  # deal with cycle
-            if ['shift'] == modifiers:
-                next = self._get_focus_next('focus_previous')
+            modifiers = set(modifiers)
+            if {'ctrl', 'alt', 'meta', 'super', 'compose'} & modifiers:
+                return False
+            if 'shift' in modifiers:
+                next = self.get_focus_previous()
             else:
-                next = self._get_focus_next('focus_next')
+                next = self.get_focus_next()
             if next:
                 self.focus = False
 
